import argparse, json, os, sys
from loguru import logger
from fsai_coco_filter.tasks.load_coco_json import load_coco_json
from fsai_coco_filter.tasks.save_coco_json import save_coco_json
from fsai_coco_filter.tasks.reset_category_ids import reset_category_ids
from fsai_coco_filter.tasks.filter_annotations import filter_annotations
from fsai_coco_filter.tasks.update_info import update_info

# The main function for execution
def main(args):

    # Load the config file
    try:
        logger.info("Loading coco json from: {}".format(args["input_file_path"]))
        coco_data = load_coco_json(args["input_file_path"])
    except Exception as e:
        logger.error(
            "Failed to load the coco json from: {}".format(args["input_file_path"])
        )
        logger.error(str(e))
        sys.exit(1)

    # Load the config file
    try:
        logger.info("Loading the config file from: {}".format(args["config_file"]))
        with open(args["config_file"], "r") as fh:
            functions = json.load(fh)
    except Exception as e:
        logger.error("Failed to load the config file: {}".format(args["config_file"]))
        logger.error(str(e))
        sys.exit(1)

    # Dynamically load the filter functions
    for func_name, func_params in functions.items():
        try:
            logger.info("Running {} with parameters: {}".format(func_name, func_params))
            coco_data = getattr(sys.modules[__name__], func_name)(
                coco_data, func_params
            )
        except Exception as e:
            logger.warning(
                "Failed to run {} with parameters: {}".format(func_name, func_params)
            )
            logger.warning(str(e))

    # Saved the filtered coco json
    try:
        logger.info("Saving coco data to: {}".format(args["output_file_path"]))
        save_coco_json(coco_data, args["output_file_path"])
    except Exception as e:
        logger.error("Failed to save coco data to: {}".format(args["output_file_path"]))
        logger.error(str(e))
        sys.exit(1)


def app():
    parser = argparse.ArgumentParser(
        description="Apply filters to a coco annotation file from a project configuration file."
    )
    parser.add_argument(
        "-i",
        "--input_file_path",
        type=str,
        dest="input_file_path",
        help="Path to the input coco annotations file.",
        required=True,
    )
    parser.add_argument(
        "-o",
        "--output_file_path",
        type=str,
        dest="output_file_path",
        help="Path to the output filtered coco annotations file.",
        required=True,
    )
    parser.add_argument(
        "-c",
        "--config_file",
        type=str,
        dest="config_file",
        help="The path to the coco filter config file",
        default=os.getcwd() + "/fcf.json",
        required=False,
    )

    args = vars(parser.parse_args())

    main(args)



import json
from echo1_coco_builder.annotations_builder import CocoAnnotationsBuilder
import pydash
from typing import List, Dict


def filter_annotations(coco_data: Dict, allowed_category_names: List) -> Dict:
    # Initialize the coco builder
    coco_builder = CocoAnnotationsBuilder()

    allowed_category_ids = []

    # Add the categories
    for c in coco_data["categories"]:
        if c["name"] in allowed_category_names:
            allowed_category_ids.append(c["id"])
            coco_builder.add_category({"id": c["id"], "name": c["name"]})

    # Add the images
    for i in coco_data["images"]:
        coco_builder.add_image(
            {
                "id": i["id"],
                "file_name": i["file_name"],
                "width": i["width"],
                "height": i["height"],
            }
        )

    # Add the annotations
    for a in coco_data["annotations"]:
        # If this is an allowed category_id
        if a["category_id"] in allowed_category_ids:
            # Then add the annotations
            coco_builder.add_annotation(
                {
                    "id": a["id"],
                    "image_id": a["image_id"],
                    "category_id": a["category_id"],
                    "bbox": a["bbox"],
                    "iscrowd": pydash.get(a, "iscrowd", 0),
                }
            )

    # Return the json as an object
    return json.loads(coco_builder.get())



import json
from operator import itemgetter
from loguru import logger
from echo1_coco_builder.annotations_builder import CocoAnnotationsBuilder
import pydash
from typing import Dict


def update_info(coco_data: Dict, update_info=True) -> Dict:

    # Initialize the coco builder
    coco_builder = CocoAnnotationsBuilder()

    # Add the categories
    for c in coco_data["categories"]:
        coco_builder.add_category({"id": c["id"], "name": c["name"]})

    # Add the images
    for i in coco_data["images"]:
        coco_builder.add_image(
            {
                "id": i["id"],
                "file_name": i["file_name"],
                "width": i["width"],
                "height": i["height"],
            }
        )

    # Add the annotations
    for a in coco_data["annotations"]:
        coco_builder.add_annotation(
            {
                "id": a["id"],
                "image_id": a["image_id"],
                "category_id": a["category_id"],
                "bbox": a["bbox"],
                "iscrowd": pydash.get(a, "iscrowd", 0),
            }
        )

        # Add the info
        coco_builder.add_info(update_info)

    # Return the json as an object
    return json.loads(coco_builder.get())



import json
from operator import itemgetter
from loguru import logger
from echo1_coco_builder.annotations_builder import CocoAnnotationsBuilder
import pydash
from typing import Dict


def reset_category_ids(coco_data: Dict, reset_category_ids=True) -> Dict:

    # Initialize the coco builder
    coco_builder = CocoAnnotationsBuilder()

    remapping_dict = {}

    # Get the categories sorted by category id
    categories = sorted(coco_data["categories"], key=itemgetter("id"), reverse=False)

    # Send a warning if the first category has an index of 0
    if categories[0]["id"] == 0:
        logger.warning("The first category already has an index of 0.")

    # Setup the remapping of the categories to being with the category id 0
    for new_category_id, c in enumerate(categories, start=0):
        old_category_id = c["id"]
        remapping_dict[old_category_id] = new_category_id
        coco_builder.add_category({"id": new_category_id, "name": c["name"]})

    # Add the images
    for i in coco_data["images"]:
        coco_builder.add_image(
            {
                "id": i["id"],
                "file_name": i["file_name"],
                "width": i["width"],
                "height": i["height"],
            }
        )

    # Add the annotations
    for a in coco_data["annotations"]:
        # If this is an allowed category_id
        if a["category_id"] in remapping_dict:
            # Get the new category id
            new_category_id = remapping_dict[a["category_id"]]

            # Then add the annotations
            coco_builder.add_annotation(
                {
                    "id": a["id"],
                    "image_id": a["image_id"],
                    "category_id": new_category_id,
                    "bbox": a["bbox"],
                    "iscrowd": pydash.get(a, "iscrowd", 0),
                }
            )

    # Return the json as an object
    return json.loads(coco_builder.get())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fsai_coco_filter', 'fsai_coco_filter.tasks']

package_data = \
{'': ['*']}

install_requires = \
['echo1-coco-builder>=0.1.8,<0.2.0', 'loguru', 'pydash>=5.1.0,<6.0.0']

entry_points = \
{'console_scripts': ['coco-filter = fsai_coco_filter.fsai_coco_filter:app']}

setup_kwargs = {
    'name': 'fsai-coco-filter',
    'version': '0.1.3',
    'description': 'Apply filters to a coco annotation file from a project configuration file.',
    'long_description': '# fsai-coco-filter\n\nfsai-coco-filter provides a way to apply filters to a coco annotation file from a project configuration file.\n\n## Installation\n```shell\n# Install fsai-coco-filter\npip install fsai-coco-filter\n```\n\n## Add an fcf.json to your project\n```json\n{\n    "filter_annotations": [\n        "Powerline Pylon, Type A, H, Y",\n        "Powerline Pylon, Type I",\n        "Light Support Structure"\n    ],\n    "reset_category_ids": true,\n    "update_info": {\n        "year": 2022,\n        "version": "v1.0",\n        "contributor": "Foundation Stack AI",\n        "description": "Contact for more info.",\n        "url": "https://fsai.dev"\n    }\n}\n```\n\n# Run the coco-filter\nfsai-coco-filter \\\n    --input_file_path /tmp/coco.json \\\n    --output_file_path /tmp/coco-filtered.json\n\n\n## coco-filter help\n```shell\ncoco-filter\n\nusage: coco-filter [-h] -i INPUT_FILE_PATH -o OUTPUT_FILE_PATH [-c CONFIG_FILE]\ncoco-filter: error: the following arguments are required: -i/--input_file_path, -o/--output_file_path\n```',
    'author': 'Michael Mohamed',
    'author_email': 'michael@foundationstack.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fsai-dev/fsai-coco-filter',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
